(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



(* ::Input::Initialization:: *)
Off[General::spell1]


(* ::Input::Initialization:: *)
Needs["ImageProcessing`"]


(* ::Input::Initialization:: *)
Needs["UnitGaussians`"]


(* ::Input::Initialization:: *)
MakeFilter::usage="MakeFilter[type,args...]: Compute a discrete two-dimensional FIR filter. Possible types are:
gauss,
gabor,
gaboranalytic (one lobe of a Gabor),
gaboranalyticoct (octave bandwidth argument),
gaussring (convolution of ring impulse and Gaussian),
gaussringoct (octave bandwidth argument).
The value is a possibly complex image in the frequency domain. The arguments, however, are typically specified in space-domain units (ie pixels). To determine arguments for a particular type, enter ??MakeFilter ";


(* ::Input::Initialization:: *)
MakeFilter[gauss,size_:16,scale_:2] :=
	Module[ {half = size/2,f1D,x},
	f1D = RotateRight[Table[uGs[x,scale/ size]//N, {x,-half,half-1}],half];
	Outer[Times,f1D,f1D]
]


(* ::Input::Initialization:: *)
MakeFilter[dog,size_:16,scale_:2,ampRatio_:1.,scaleRatio_:3.] :=
	Module[ {half = size/2,c1D,s1D,x},
	c1D = RotateRight[Table[uGs[x,scale/ size]//N
	 ,{x,-half,half-1}],half];
	s1D = ampRatio RotateRight[Table[
	 uGs[x,scaleRatio scale/ size]//N
	 ,{x,-half,half-1}],half];
	Outer[Times,c1D,c1D] - Outer[Times,s1D,s1D]
]


(* ::Input::Initialization:: *)
MakeFilter[gabor,size_:16,scale_:8,freq_:4,orient_:0,phase_:0] :=
	Module[ {u,v,half = size/2,x,cu,cv,su,sv,gu,gv},
	u = freq Cos[orient] 2 Pi /size;
	v = freq Sin[orient] 2 Pi /size;
	gu = Table[ N[ ugs[x,scale] ],{x,-half,half-1}];
	gv := gu; (* temporary expedient *)
	cu = InverseFourier[RotateRight[
		 gu Table[N[ Cos[u x] ],{x,-half,half-1}],half]];
	cv = InverseFourier[RotateRight[
		 gv Table[N[ Cos[v x] ],{x,-half,half-1}],half]];
	su = InverseFourier[RotateRight[
		 gu Table[N[ Sin[u x] ],{x,-half,half-1}],half]];
	sv = InverseFourier[RotateRight[
		 gv Table[N[ Sin[v x] ],{x,-half,half-1}],half]];
Chop[2 size * (
 N[Cos[phase]] (Outer[Times,cv,cu] -  
 Outer[Times,sv,su]) -
 N[Sin[phase]] (Outer[Times,cu,sv] +  
 Outer[Times,cv,su]) )]
] 


(* ::Input::Initialization:: *)
MakeFilter[gaussring,size_:16,freq_:1,scale_:2] := Module[
 {half = size/2,x,y,f,r,a,pr2},
 f = N[2 Pi freq/size];
 a = N[f Pi scale^-2];
 pr2 = N[ - Pi / (scale^2.)];
	Chop[InverseFourier[ WrapImage[ a Table[
	(r= N[Abs[x + I y]];
	 N[BesselJ[0,f r ] Exp[pr2 (r^2.)]] )
	 , {y,-half,half-1} , {x,-half,half-1} ]]]]]


(* ::Input::Initialization:: *)
MakeFilter[gaussringoct,size_:16,freq_:1,octaves_:1] :=
 MakeFilter[gaussring,size,freq
   ,size oct2scale[freq,octaves]]


(* ::Input::Initialization:: *)
(* obsolete, retained for backward compatibility *)
makeGaussRingFilter[size_:16,freq_:1,scale_:2] := Module[
 {half = size/2,x,y,f,r},
 f = N[2 Pi freq/size];
	Chop[N[Table[
	(r = Abs[x + I y];
	 f Pi BesselJ[0,f r ] ugs[r,scale] / scale)
	 , {x,-half,half-1}, {y,-half,half-1}  ]]]]


(* ::Input::Initialization:: *)
(* obsolete, retained for backward compatibility *)
makeGaussRingFilterOct[size_:16,freq_:1,octaves_:1] :=
 makeGaussRingFilter[size,freq
   ,size oct2scale[freq,octaves]]


(* ::Input::Initialization:: *)
MakeFilter[gaboranalytic
 ,size_:16,scale_:8,freq_:4,orient_:0 ,phase_:0] := Module[{u,v,half = size/2,x,gu,gv,iscale=scale/size},
 {u,v} = freq {Cos[#],Sin[#]}&[orient] ;
 gu = RotateRight[Table[ N[ uGs[x-u,iscale] ]
   ,{x,-half,half-1}],half];
	gv = RotateRight[Table[ N[ uGs[x-v,iscale] ]
	  ,{x,-half,half-1}],half];
Chop[Outer[Times,gv,gu]] 
] 


(* ::Input::Initialization:: *)
MakeFilter[gaboranalyticoct
 ,size_:16,oct_:1,freq_:4,orient_:0 ,phase_:0] :=
	MakeFilter[gaboranalytic, size, size * oct2scale[freq,oct] , 
		freq, orient];


(* ::Input::Initialization:: *)
MakeFilter[gaboranalyticoct
 ,size_,freqOct_,freq_,orient_,phase_,aspectRatio_]:=
	Module[{half=size/2},
		Map[RotateRight[#,half]&,RotateRight[Table[N[
						uGs[-freq+x*Cos[orient]+y*Sin[orient],
							oct2scale[freq,freqOct]]*
						uGs[y*Cos[orient]-x*Sin[orient],
							oct2scale[freq,freqOct]*aspectRatio]],
					{y,-half,half-1},{x,-half,half-1}],half]]]


(* ::Input::Initialization:: *)
MakeFilter[LogParabola,size_:16,f0_:2,bw_:1.
  ,dc_:0.]:= Module[ {half = size/2,f,u,v},
	WrapImage[ Table[
	 (f = Abs[u + I v]//N;
	 If[f==0.,dc,10.^(-((Log[10.,f/f0]/bw)^2.))]//N)
	 ,{u,-half,half-1}
	 ,{v,-half,half-1}]]]


(* ::Input::Initialization:: *)
MakeFilter[gaboranalytic3D
 ,sizes_:{16,16,8},scales_:{8,8,4},freqs_:{4,4,2}
 ,phase_:0] := Module[{u,v,half = sizes/2,x,gu,gv,iscale=scales/sizes},
 {gu,gv,gw} = Table[
 RotateRight[Table[
  N[ uGs[x-freqs[[i]],iscale[[i]]] ]
   ,{x,-half[[i]],half[[i]]-1}],half[[i]]],{i,3}];
Chop[Outer[Times,gw,gv,gu]] 
] 


(* ::Input::Initialization:: *)
MakeFilter[fractal,size_:16,pow_:1] :=
 Module[ {half = size/2},
	WrapImage[Chop[ N[ Table[ Max[1,Abs[x + I y]]^(-pow)
	 , {y,-half,half-1} , {x,-half,half-1} ] ] ] ]]



