(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



<< Psychometrica.m


QuestExperiment::usage = "QuestExperiment[guesses, trials]: Conducts an experiment using the QUEST adaptive procedure. The user must specify a list of an initial estimates of threshold for each condition. The function will conduct trials trials for each condition. The function itself calls a function f[condition,strength] (where f is the value of the option QuestResponse) which presents a stimulus and collects a response, and which serves as the interface to the device and stimulus-dependent parts of the experiment. QuestExperiment returns a structure of the form {sequence,strengths,functions, parameters}, where sequence is the complete trial sequence, each trial specified by a triple {condition, strength, response}. strengths, functions, and parameters are all lists, with lengths equal to the number of conditions. Each element of strengths is a list of possible testing strengths. Each element of functions is an assumed psychometric function, and parameters are their associated parameters. If a response of -1 is received during the main trial loop, the loop is exited and the existing data are returned. 
Options:
QuestResponse->SimulatedObserver: function defining the simulated observer
QuestFunction->LogWeibullPF: assumed psychometric function
QuestParameters->{3.5,.5,.01}: assumed psychometric parameters
QuestMethod\[Rule]\"Mean\": whether mode, median, or mean of the posterior probability function will be used as the testing point
QuestPlotShow->False:  whether a plot will be produced after each trial, which is useful for tutorial purposes. 
QuestSigma->100: standard deviation (in dB) of the normal prior probability density
QuestRange->{-60,0,2}: minimum, maximum and increment of the list of possible testing points
QuestSeed->Automatic: if a number, will be used as the random seed (determines order of presentation of conditions, order within forced-choice presentations, and behaviour of the simulated observer)
QuestScatter->0: 
TargetProbabilities\[Rule]Automatic: can be set to a list of target probabilities, amongst which QuestExperiment will choose pseudorandomly; the automatic setting is equivalent to {QuestFunction[alpha,{alpha,#}&@@QuestParameters]}
DeltaCheckFreq\[Rule]0: determines the frequency with which the largest strength (QuestRange[[2]]) will be presented
GammaCheckFreq\[Rule]0: determines the frequency with which the smallest strength (QuestRange[[1]]) will be presented
SaveQuestMovie\[Rule]False: whether to save the sequence of plots as the global variable QuestMovie.
Prior data returned from QuestExperiment can be used to initialize a new session by entering it in place of the guesses argument.";


Options[QuestExperiment] = {QuestFunction->LogWeibullPF, QuestParameters->{3.5,.5,.01}, QuestMethod->"Mean", QuestPlotShow->False, QuestSigma->100, QuestRange->{-60,0,2}, QuestSeed->Automatic, QuestResponse->SimulatedObserver,QuestScatter->0,TargetProbabilities->Automatic,DeltaCheckFreq->0,GammaCheckFreq->0,SaveQuestMovie->False};


QuestExperiment[guesses_, trials_, opts___Rule] :=
 Module[ {c,i=1,getResponse, plot, method, range, sigma, seed, pfunction, params, targetps,dcf,gcf,order, strength, strengths, response, sequence={}, tables, functions, parameters, data, nc, first,catch},
{getResponse, plot, method, range, sigma, seed, pfunction, params,targetps,dcf,gcf} ={QuestResponse, QuestPlotShow, QuestMethod, QuestRange, QuestSigma, QuestSeed,
QuestFunction, QuestParameters,TargetProbabilities,DeltaCheckFreq,GammaCheckFreq} /. {opts} /. Options[QuestExperiment];

If[NumberQ[seed],SeedRandom[seed]];

data := {sequence,strengths,tables,functions,parameters};

If[Depth[guesses] < 3,
	(
	nc = Length[guesses];
	first = Table[True,{nc}];
	{strengths,functions,parameters} = Transpose[Table[{Range @@ range,pfunction,params},{nc}]];
 	tables = MapThread[
    	LogGauss[#1,sigma,#2]&,{strengths,guesses}];
 	),
 	(
 	nc = Length[guesses[[2]]];
 	{sequence,strengths,tables,functions,parameters}=guesses;
 	first = Table[False,{nc}];
 	)];	
 	
	order = RandomSample[Flatten[ Table[c,{c,nc}
	 ,{If[ListQ[trials],trials[[c]],trials]}]]];

Catch[Do[
c=order[[i]];
strength = If[first[[c]]
,(first[[c]] = False;guesses[[c]])
,strengths[[c,NextIndex[tables[[c]],method,opts]]]];

If[ListQ[targetps],(Off[Solve::ifun];
strength=Solve[RandomChoice[targetps]==pfunction[dummy,({strength,##1}&)@@params],dummy][[1,1,2]];
On[Solve::ifun];
strength=First[Nearest[strengths[[c]],strength,1]])];

catch=RandomReal[];
If[catch<dcf,strength=Max[strengths[[c]]]
,If[catch<dcf+gcf,strength=Min[strengths[[c]]]]];

 response = getResponse[c,strength];
If[response==-1,Throw[data]];
{sequence,tables}=QuestUpdate[c,strength,response,data];
If[plot, Print@QuestPlot[data,c,opts]];
  ,{i,Length[order]}]];
data]


QuestExperiment[guesses_, trials_, opts___Rule] :=
 Module[ {c,i=1,getResponse, plot, method, range, sigma, seed, pfunction, params, targetps,dcf,gcf,order, strength, strengths, response, sequence={}, tables, functions, parameters, data, nc, first,catch,movieq,movie={},qp},
{getResponse, plot, method, range, sigma, seed, pfunction, params,targetps,dcf,gcf,movieq} ={QuestResponse, QuestPlotShow, QuestMethod, QuestRange, QuestSigma, QuestSeed,
QuestFunction, QuestParameters,TargetProbabilities,DeltaCheckFreq,GammaCheckFreq,SaveQuestMovie} /. {opts} /. Options[QuestExperiment];

If[movieq,QuestMovie={}];
If[NumberQ[seed],SeedRandom[seed]];

data := {sequence,strengths,tables,functions,parameters};

If[Depth[guesses] < 3,
	(
	nc = Length[guesses];
	first = Table[True,{nc}];
	{strengths,functions,parameters} = Transpose[Table[{Range @@ range,pfunction,params},{nc}]];
 	tables = MapThread[
    	LogGauss[#1,sigma,#2]&,{strengths,guesses}];
 	),
 	(
 	nc = Length[guesses[[2]]];
 	{sequence,strengths,tables,functions,parameters}=guesses;
 	first = Table[False,{nc}];
 	)];	
 	
	order = RandomSample[Flatten[ Table[c,{c,nc}
	 ,{If[ListQ[trials],trials[[c]],trials]}]]];

Catch[Do[
c=order[[i]];
strength = If[first[[c]]
,(first[[c]] = False;guesses[[c]])
,strengths[[c,NextIndex[tables[[c]],method,opts]]]];

If[ListQ[targetps],(Off[Solve::ifun];
strength=Solve[RandomChoice[targetps]==pfunction[dummy,({strength,##1}&)@@params],dummy][[1,1,2]];
On[Solve::ifun];
strength=First[Nearest[strengths[[c]],strength,1]])];

catch=RandomReal[];
If[catch<dcf,strength=Max[strengths[[c]]]
,If[catch<dcf+gcf,strength=Min[strengths[[c]]]]];

 response = getResponse[c,strength];
If[response==-1,Throw[data]];
{sequence,tables}=QuestUpdate[c,strength,response,data];
If[plot, Print@(qp=QuestPlot[data,c,opts])];
If[movieq, AppendTo[QuestMovie,qp]];
  ,{i,Length[order]}]];
data]


QuestUpdate::usage = "QuestUpdate[condition,strength,response,qdata]: This function is called after a trial of condition at strength with outcome response (0 or 1). It updates and returns the structure qdata, which is a complete record of the experiment. The structure of qdata is {QSequence, QStrengths, QTables, QFunctions, QParameters}. QSequence is a list of triples, each of the form {condition, strength, response} which describes the trial sequence in order of occurrence. QStrengths is a list of lists, one per condition, each specifying the available strengths. QTables is a list of lists, one per condition, each specifying the current un-normalized, log posterior probability of threshold at each strength. QFunctions is a list of function names, one per condition, each specifying the psychometric function used by QUEST. QParameters is a list of lists, one per condition, each specifying the parameters assigned to the corresponding psychometric function.";


QuestUpdate[condition_,strength_,response_
,{QSequence_,QStrengths_,QTables_,QFunctions_,QParameters_} ]:=
{Append[QSequence,{condition,strength,response}],
 MapAt[ Function[x, x + Check[Log[ If[response==1,#,1-#]&[
QFunctions[[condition]][strength ,Prepend[QParameters[[condition]],#]]& /@
 QStrengths[[condition]]]],-100]],QTables,condition]}


QuestPlot::usage = "QuestPlot[data, condition]: Plots several summaries of the progress of a Quest session for one condition. QuestPlot[data] plots separate summaries for all conditions. Included on the same figure are 1) the normalized antilog of the Quest function, which is proportional to the posterior probability density, 2) the histogram of trial placements, 3) the sequence of trial placements and their individual outcomes (green for yes/correct, red for no/incorrect), and 4) the proportion correct, the best-fitting psychometric function, and the printed estimates of threshold, slope, and error. Options to ListPlot and PsychometricPlot can also be included. The argument data is the basic data structure maintained by Quest (see QuestUpdate) and returned by QuestExperiment. The various components of the plot may be toggled by means of the following options, shown with their default strengths: Fit->False, Density->True, Histogram->True, Trials->True.";


Options[QuestPlot] = {Fit->False, Density->True, Histogram->True, Trials->True};


QuestPlot[{QSequence_,QStrengths_,QTables_,QFunctions_,QParameters_},condition_Integer,opts___Rule]:=Module[
{showfit,histogram,density,trials,cSequence,trial,vscale,xrange,pts,f1,tab,f2,f3,f4,fit,gray},{showfit,histogram,density,trials}=
{Fit,Histogram,Density,Trials}/.  {opts}/.  Options[QuestPlot];cSequence=Cases[QSequence,{condition,__}];
trial=Length[cSequence];
vscale=1/Max[32,trial];
xrange=MinMax[QStrengths[[condition]]]+0.5` {-1,1};
pts=MapIndexed[{#1[[2]],#2[[1]] vscale}&,cSequence];tab=({#1[[1]],vscale Plus@@#1[[{2,3}]]}&)/@SequenceToTable[cSequence][[condition]];
gray =GrayLevel[If[density,.85,1]];f1=ListLinePlot[Transpose[{QStrengths[[condition]],E^(#1-Max[#1]&)[QTables[[condition]]]}]
,Filling->0,PlotRange->All,PlotStyle->{gray,AbsoluteThickness[0]}
,FillingStyle->{gray}];
f2=If[histogram,BarChartABW[tab,.9],{}];
f3=If[trials,Graphics[{Line[pts],PointSize[0.025]
,Transpose[{(Hue[0.35 #1[[3]]]&)/@cSequence,Point/@pts}]}],{}];f4=If[!showfit,{},(fit=PsychometricFit[SequenceToTable[QSequence][[condition]],opts,PsychometricFunction->QFunctions[[condition]],PsychometricParameters->Prepend[QParameters[[condition]],Automatic]];If[NumberQ[fit[[3,2]]]
,PsychometricPlot[fit,PlotRange->{xrange,{0,1}},Sequence@@FilterRules[{opts},Options[PsychometricPlot]]],{}])];
Show[f1,f2,f3,f4,Sequence@@FilterRules[{opts},Options[Graphics]]
,Frame->True
,Axes->False
,FrameTicks->{Automatic ,Automatic,Automatic,{{vscale trial,trial}}}
,PlotLabel->"Condition "<>ToString[condition]<>" Trial "<>ToString[trial]
,FrameLabel->{"Strength","Probability",None,"Trials"}
,PlotRange->{xrange,{0,1.01}}
,PsychometricaStyles]]


QuestPlot[data_,opts___Rule] :=
 Table[QuestPlot[data,c,opts],{c,Length[data[[2]]]}]


BarChartABW[data_,width_] :=
Graphics[{Opacity[.5],Blue,EdgeForm[]
,Rectangle[{#[[1]]-width/2,0},{#[[1]]+width/2,#[[2]]}]& /@ data}]


QuestMean::usage = "QuestMean[QTable]: Given a QTable (a list of un-normalized, log posterior probabilities of threshold strength), QuestMean finds the index nearest to the approximate mean of the posterior density.";


QuestMean[qTable_] := Module[{pdf},
 pdf = Exp[qTable];
 pdf = pdf / (Plus @@ pdf);
 Round[Plus @@ (pdf Range[Length[pdf]])]]


QuestMedian::usage = "QuestMedian[QTable]: Given a QTable (a list of un-normalized, log posterior probabilities of threshold strength), QuestMedian finds the index nearest to the approximate median of the posterior density.";


QuestMedian[qTable_] := Module[{pdf,half},
 pdf = Exp[qTable];
half = 0.5 ( Plus @@ pdf);
IndexMax[-Abs[Drop[FoldList[Plus,-half,pdf],1]]]]


QuestMode::usage = "QuestMode[QTable]: Given a QTable (a list of un-normalized, log posterior probabilities of threshold strength), QuestMode finds the index of the mode of the posterior density.";


QuestMode := IndexMax


NextIndex::usage = "NextIndex[qTable_, method_]: Selects the next index for trial placement, based on the desired method (\"Mode\", \"Median\", or \"Mean\")in the Quest psychometric procedure.qTable is a list of log un-normalized posterior probabilities of threshold location.";


NextIndex[qTable_,method_,opts___]:=Module[{ni,si},
ni=Switch[method,
"Mode",QuestMode,
"Mean",QuestMean,
"Median",QuestMedian][qTable];si=Clip[ni+RandomInteger[(QuestScatter/.  {opts} /.  Options[QuestExperiment]) {-1,1}],{1,Length[Range@@(QuestRange/. {opts}/. Options[QuestExperiment])]}];si]


LogGauss::usage = "LogGauss[strength,sigma,mean]: This function is used to set a prior probability density for each condition, which may be done by initializing the log likelihood list (QTable). It is the log of  a Gaussian (a parabola). The arguments are a strength, a standard deviation of the Gaussian, and a mean of the Gaussian.";


LogGauss[strength_,sigma_,mean_] :=-Log[sigma Sqrt[2. Pi] ] - ( 0.5 ((strength-mean)/sigma)^2)


SimulatedObserver::usage = "SimulatedObserver[condition,strength]: Simulates a trial for condition at strength. It returns 1 or 0 for success or failure, respectively. By default, it obeys a LogWeibullPF psychometric function with parameters {-20,3.5,.5,.01,0.}.These may be changed via SetOptions and the option names PsychometricFunction and PsychometricParameters. ";


Options[SimulatedObserver] = {PsychometricFunction->LogWeibullPF, PsychometricParameters->{-20,4.,.5,.01}};


SimulatedObserver[condition_,strength_]:=If[(PsychometricFunction/.  Options[SimulatedObserver])[strength,PsychometricParameters/.  Options[SimulatedObserver]]>RandomReal[],1,0]



